package com.jessebrault.ssg

import com.jessebrault.ssg.buildscript.DefaultBuildScriptConfiguratorFactory
import com.jessebrault.ssg.buildscript.SimpleBuildScriptRunner
import com.jessebrault.ssg.util.Diagnostic
import org.apache.logging.log4j.LogManager
import org.apache.logging.log4j.Logger
import picocli.CommandLine

abstract class AbstractBuildCommand extends AbstractSubCommand {

    private static final Logger logger = LogManager.getLogger(AbstractBuildCommand)

    @CommandLine.Option(
            names = '--baseDir',
            description = 'The base directory for all components.'
    )
    File baseDir = new File('.')

    @CommandLine.Option(
            names = ['-s', '--script', '--buildScript'],
            description = 'The build script file to execute, relative to the baseDir.'
    )
    File buildScript = new File('ssgBuilds.groovy')

    @CommandLine.Option(
            names = '--scriptArgs',
            description = 'Named argument(s) to pass directly to the build script.',
            split = ','
    )
    Map<String, String> scriptArgs = [:]

    @CommandLine.Option(
            names = '--buildSrcDirs',
            description = 'Path(s) to director(ies) containing Groovy classes and scripts which should be visible to the main build script, relative to the baseDir.',
            split = ',',
            paramLabel = 'buildSrcDir'
    )
    Collection<File> buildSrcDirs = [new File('buildSrc')]

    @CommandLine.Option(
            names = ['-b', '--build'],
            description = 'The name of a build to execute.',
            paramLabel = 'buildName'
    )
    Collection<String> requestedBuilds = ['default']

    protected StaticSiteGenerator staticSiteGenerator = null

    protected final Integer doSingleBuild(String requestedBuild) {
        logger.traceEntry('requestedBuild: {}', requestedBuild)

        if (this.staticSiteGenerator == null) {
            this.staticSiteGenerator = new BuildScriptBasedStaticSiteGenerator(
                    new SimpleBuildScriptRunner(),
                    new DefaultBuildScriptConfiguratorFactory(this.baseDir),
                    this.buildScript == new File('ssgBuilds.groovy') || this.buildScript.exists()
                            ? new File(this.baseDir, this.buildScript.path)
                            : null,
                    this.buildSrcDirs.collect { new File(this.baseDir, it.path) },
                    this.scriptArgs
            )
        }

        final Collection<Diagnostic> diagnostics = []
        if (!this.staticSiteGenerator.doBuild(requestedBuild, diagnostics.&addAll)) {
            diagnostics.each { logger.warn(it) }
            logger.traceExit(1)
        } else {
            logger.traceExit(0)
        }
    }

}
